package dev.fitko.fitconnect.core.util;

import com.nimbusds.jose.util.Base64;
import dev.fitko.fitconnect.api.exceptions.RootCertificateException;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.Objects.*;

public class CertificateLoader {

    public static CertificateLoader create() {
        return new CertificateLoader();
    }

    public List<String> loadTrustedRootCertificates(final String directoryPath) {
        return loadCertificatesFromDirectory(directoryPath)
                .stream()
                .map(this::convertToBase64EncodedX509Certificate)
                .collect(Collectors.toList());
    }

    private List<String> loadCertificatesFromDirectory(final String directoryPath) {
        final File folder = new File(requireNonNull(CertificateLoader.class.getClassLoader().getResource(directoryPath)).getFile());
        return Arrays.stream(requireNonNull(folder.listFiles())).map(readCertFileToString()).collect(Collectors.toList());
    }
    private String convertToBase64EncodedX509Certificate(final String certificateString) {
        final InputStream certificateInputStream = new ByteArrayInputStream(certificateString.getBytes());
        try {
            final CertificateFactory certificateFactory = CertificateFactory.getInstance("X.509");
            final X509Certificate x509Certificate = (X509Certificate) certificateFactory.generateCertificate(certificateInputStream);
            return Base64.encode(x509Certificate.getEncoded()).toString();
        } catch (final CertificateException e) {
            throw new RootCertificateException(e);
        }
    }

    private Function<File, String> readCertFileToString() {
        return file -> {
            try {
                return Files.readString(file.toPath());
            } catch (final IOException e) {
                throw new RootCertificateException(e);
            }
        };
    }
}
